@@ -21,9 +21,9 @@ fn feature_status(feature: &str) -> String {
     ))
     .is_some()
     {
-        format!("+{}", feature)
+        format!("+{feature}")
     } else {
-        format!("-{}", feature)
+        format!("-{feature}")
     }
 }
 
@@ -36,22 +36,27 @@ fn features() -> String {
 }
 
 fn main() {
-    for dir in ["assets", "assets/basic", "assets/large"] {
-        println!("cargo:rerun-if-changed={}", dir);
+    for dir in [
+        "assets/syntax",
+        "assets/syntax/basic",
+        "assets/syntax/large",
+        "assets/themes",
+    ] {
+        println!("cargo:rerun-if-changed={dir}");
         for entry in read_dir(dir).unwrap() {
             let path = entry.unwrap().path();
             let path = path.to_str().unwrap();
             if path.ends_with(".sublime-syntax") || path.ends_with(".tmTheme") {
-                println!("cargo:rerun-if-changed={}", path);
+                println!("cargo:rerun-if-changed={path}");
             }
         }
     }
 
-    build_syntax("assets/basic", "basic.packdump");
-    build_syntax("assets/large", "large.packdump");
+    build_syntax("assets/syntax/basic", "basic.packdump");
+    build_syntax("assets/syntax/large", "large.packdump");
 
     let out_dir = env::var_os("OUT_DIR").unwrap();
-    let ts = ThemeSet::load_from_folder("assets").unwrap();
+    let ts = ThemeSet::load_from_folder("assets/themes").unwrap();
     dump_to_file(&ts, Path::new(&out_dir).join("themepack.themedump")).unwrap();
 
     println!("cargo:rustc-env=XH_FEATURES={}", features());
