@@ -60,7 +60,7 @@ pub struct Cli {
     /// Like --form, but force a multipart/form-data request even without files.
     ///
     /// Overrides both --json and --form.
-    #[clap(long, conflicts_with = "raw", overrides_with_all = &["json", "form"])]
+    #[clap(long, conflicts_with_all = &["raw", "compress"], overrides_with_all = &["json", "form"])]
     pub multipart: bool,
 
     /// Pass raw request data without extra processing.
@@ -182,6 +182,17 @@ Example: --print=Hb"
     #[clap(short = 'S', long = "stream", name = "stream")]
     pub stream_raw: bool,
 
+    ///  Content compressed (encoded) with Deflate algorithm.
+    ///
+    ///  The Content-Encoding header is set to deflate.
+    ///
+    ///  Compression is skipped if it appears that compression ratio is negative.
+    ///  Compression can be forced by repeating this option.
+    ///
+    ///  Note: Compression cannot be used if the Content-Encoding request header is present.  
+    #[clap(short = 'x', long = "compress", name = "compress", action = ArgAction::Count)]
+    pub compress: u8,
+
     #[clap(skip)]
     pub stream: Option<bool>,
 
@@ -212,7 +223,7 @@ Example: --print=Hb"
     #[clap(long, value_name = "FILE")]
     pub session: Option<OsString>,
 
-    /// Create or read a session without updating it form the request/response exchange.
+    /// Create or read a session without updating it from the request/response exchange.
     #[clap(long, value_name = "FILE", conflicts_with = "session")]
     pub session_read_only: Option<OsString>,
 
@@ -347,6 +358,12 @@ Example: --print=Hb"
     #[clap(short = '6', long)]
     pub ipv6: bool,
 
+    /// Connect using a Unix domain socket.
+    ///
+    /// Example: xh :/index.html --unix-socket=/var/run/temp.sock
+    #[clap(long, value_name = "FILE")]
+    pub unix_socket: Option<PathBuf>,
+
     /// Do not attempt to read stdin.
     ///
     /// This disables the default behaviour of reading the request body from stdin
@@ -367,6 +384,27 @@ Example: --print=Hb"
     #[clap(long)]
     pub curl_long: bool,
 
+    /// Generate shell completions or man pages.
+    #[arg(
+        long,
+        value_name = "KIND",
+        hide_possible_values = true,
+        long_help = "\
+Generate shell completions or man pages. Possible values are:
+
+    complete-bash
+    complete-elvish
+    complete-fish
+    complete-nushell
+    complete-powershell
+    complete-zsh
+    man
+
+Example: xh --generate=complete-bash > xh.bash",
+        conflicts_with = "raw_method_or_url"
+    )]
+    pub generate: Option<Generate>,
+
     /// Print help.
     #[clap(long, action = ArgAction::HelpShort)]
     pub help: Option<bool>,
@@ -381,8 +419,8 @@ Example: --print=Hb"
     ///
     /// A leading colon works as shorthand for localhost. ":8000" is equivalent
     /// to "localhost:8000", and ":/path" is equivalent to "localhost/path".
-    #[clap(value_name = "[METHOD] URL")]
-    raw_method_or_url: String,
+    #[clap(value_name = "[METHOD] URL", required = true)]
+    raw_method_or_url: Option<String>,
 
     /// Optional key-value pairs to be included in the request.
     ///
@@ -424,8 +462,7 @@ Example: --print=Hb"
     ///
     /// A backslash can be used to escape special characters, e.g. "weird\:key=value".
     ///
-    /// To construct a complex JSON object, the REQUEST_ITEM's key can be set to a JSON path instead of a field name.
-    /// For more information on this syntax, refer to https://httpie.io/docs/cli/nested-json.
+    /// To construct a complex JSON object, the REQUEST_ITEM's key can be set to a JSON path instead of a field name. For more information on this syntax, refer to https://httpie.io/docs/cli/nested-json.
     #[clap(value_name = "REQUEST_ITEM", verbatim_doc_comment)]
     raw_rest_args: Vec<String>,
 
@@ -480,21 +517,28 @@ impl Cli {
         let matches = app.try_get_matches_from_mut(iter)?;
         let mut cli = Self::from_arg_matches(&matches)?;
 
-        match cli.raw_method_or_url.as_str() {
-            "help" => {
-                // opt-out of clap's auto-generated possible values help for --pretty
-                // as we already list them in the long_help
-                app = app.mut_arg("pretty", |a| a.hide_possible_values(true));
+        app.get_bin_name()
+            .and_then(|name| name.split('.').next())
+            .unwrap_or("xh")
+            .clone_into(&mut cli.bin_name);
 
-                app.print_long_help().unwrap();
-                safe_exit();
-            }
-            "generate-completions" => return Err(generate_completions(app, cli.raw_rest_args)),
-            "generate-manpages" => return Err(generate_manpages(app, cli.raw_rest_args)),
-            _ => {}
+        if cli.generate.is_some() {
+            return Ok(cli);
         }
+
+        let mut raw_method_or_url = cli.raw_method_or_url.clone().unwrap();
+
+        if raw_method_or_url == "help" {
+            // opt-out of clap's auto-generated possible values help for --pretty
+            // as we already list them in the long_help
+            app = app.mut_arg("pretty", |a| a.hide_possible_values(true));
+
+            app.print_long_help().unwrap();
+            safe_exit();
+        }
+
         let mut rest_args = mem::take(&mut cli.raw_rest_args).into_iter();
-        let raw_url = match parse_method(&cli.raw_method_or_url) {
+        let raw_url = match parse_method(&raw_method_or_url) {
             Some(method) => {
                 cli.method = Some(method);
                 rest_args.next().ok_or_else(|| {
@@ -506,7 +550,7 @@ impl Cli {
             }
             None => {
                 cli.method = None;
-                mem::take(&mut cli.raw_method_or_url)
+                mem::take(&mut raw_method_or_url)
             }
         };
         for request_item in rest_args {
@@ -517,11 +561,6 @@ impl Cli {
             );
         }
 
-        app.get_bin_name()
-            .and_then(|name| name.split('.').next())
-            .unwrap_or("xh")
-            .clone_into(&mut cli.bin_name);
-
         if matches!(cli.bin_name.as_str(), "https" | "xhs" | "xhttps") {
             cli.https = true;
         }
@@ -536,7 +575,7 @@ impl Cli {
         cli.url = construct_url(&raw_url, cli.default_scheme.as_deref()).map_err(|err| {
             app.error(
                 clap::error::ErrorKind::ValueValidation,
-                format!("Invalid <URL>: {}", err),
+                format!("Invalid <URL>: {err}"),
             )
         })?;
 
@@ -614,8 +653,8 @@ impl Cli {
             .filter(|a| !a.is_positional())
             .map(|opt| {
                 let long = opt.get_long().expect("long option");
-                clap::Arg::new(format!("no-{}", long))
-                    .long(format!("no-{}", long))
+                clap::Arg::new(format!("no-{long}"))
+                    .long(format!("no-{long}"))
                     .hide(true)
                     .action(ArgAction::SetTrue)
                     // overrides_with is enough to make the flags take effect
@@ -625,9 +664,12 @@ impl Cli {
             })
             .collect();
 
-        app.args(negations)
+        let mut app = app.args(negations)
             .after_help(format!("Each option can be reset with a --no-OPTION argument.\n\nRun \"{} help\" for more complete documentation.", env!("CARGO_PKG_NAME")))
-            .after_long_help("Each option can be reset with a --no-OPTION argument.")
+            .after_long_help("Each option can be reset with a --no-OPTION argument.");
+
+        app.build();
+        app
     }
 
     pub fn logger_config(&self) -> env_logger::Builder {
@@ -733,220 +775,17 @@ fn construct_url(
         // Allow users to quickly convert a URL copied from a clipboard to xh/HTTPie command
         // by simply adding a space before `://`.
         // Example: https://example.org -> https ://example.org
-        format!("{}{}", default_scheme, url).parse()?
+        format!("{default_scheme}{url}").parse()?
     } else if url.starts_with(':') {
         format!("{}{}{}", default_scheme, "localhost", url).parse()?
     } else if !Regex::new("[a-zA-Z0-9]://.+").unwrap().is_match(url) {
-        format!("{}{}", default_scheme, url).parse()?
+        format!("{default_scheme}{url}").parse()?
     } else {
         url.parse()?
     };
     Ok(url)
 }
 
-#[cfg(feature = "man-completion-gen")]
-// This signature is a little weird: we either return an error or don't return at all
-fn generate_completions(mut app: clap::Command, rest_args: Vec<String>) -> clap::error::Error {
-    let bin_name = app.get_bin_name().unwrap().to_string();
-    if rest_args.len() != 1 {
-        return app.error(
-            clap::error::ErrorKind::WrongNumberOfValues,
-            "Usage: xh generate-completions <DIRECTORY>",
-        );
-    }
-
-    for &shell in clap_complete::Shell::value_variants() {
-        // Elvish complains about multiple deprecations and these don't seem to work
-        if shell != clap_complete::Shell::Elvish {
-            clap_complete::generate_to(shell, &mut app, &bin_name, &rest_args[0]).unwrap();
-        }
-    }
-    safe_exit();
-}
-
-#[cfg(feature = "man-completion-gen")]
-fn generate_manpages(mut app: clap::Command, rest_args: Vec<String>) -> clap::error::Error {
-    use roff::{bold, italic, roman, Roff};
-    use time::OffsetDateTime as DateTime;
-
-    if rest_args.len() != 1 {
-        return app.error(
-            clap::error::ErrorKind::WrongNumberOfValues,
-            "Usage: xh generate-manpages <DIRECTORY>",
-        );
-    }
-
-    let items: Vec<_> = app.get_arguments().filter(|i| !i.is_hide_set()).collect();
-
-    let mut request_items_roff = Roff::new();
-    let request_items = items
-        .iter()
-        .find(|opt| opt.get_id() == "raw_rest_args")
-        .unwrap();
-    let request_items_help = request_items
-        .get_long_help()
-        .or_else(|| request_items.get_help())
-        .expect("request_items is missing help")
-        .to_string();
-
-    // replace the indents in request_item help with proper roff controls
-    // For example:
-    //
-    // ```
-    // normal help normal help
-    // normal help normal help
-    //
-    //   request-item-1
-    //     help help
-    //
-    //   request-item-2
-    //     help help
-    //
-    // normal help normal help
-    // ```
-    //
-    // Should look like this with roff controls
-    //
-    // ```
-    // normal help normal help
-    // normal help normal help
-    // .RS 12
-    // .TP
-    // request-item-1
-    // help help
-    // .TP
-    // request-item-2
-    // help help
-    // .RE
-    //
-    // .RS
-    // normal help normal help
-    // .RE
-    // ```
-    let lines: Vec<&str> = request_items_help.lines().collect();
-    let mut rs = false;
-    for i in 0..lines.len() {
-        if lines[i].is_empty() {
-            let prev = lines[i - 1].chars().take_while(|&x| x == ' ').count();
-            let next = lines[i + 1].chars().take_while(|&x| x == ' ').count();
-            if prev != next && next > 0 {
-                if !rs {
-                    request_items_roff.control("RS", ["8"]);
-                    rs = true;
-                }
-                request_items_roff.control("TP", ["4"]);
-            } else if prev != next && next == 0 {
-                request_items_roff.control("RE", []);
-                request_items_roff.text(vec![roman("")]);
-                request_items_roff.control("RS", []);
-            } else {
-                request_items_roff.text(vec![roman(lines[i])]);
-            }
-        } else {
-            request_items_roff.text(vec![roman(lines[i].trim())]);
-        }
-    }
-    request_items_roff.control("RE", []);
-
-    let mut options_roff = Roff::new();
-    let non_pos_items = items
-        .iter()
-        .filter(|a| !a.is_positional())
-        .collect::<Vec<_>>();
-
-    for opt in non_pos_items {
-        let mut header = vec![];
-        if let Some(short) = opt.get_short() {
-            header.push(bold(format!("-{}", short)));
-        }
-        if let Some(long) = opt.get_long() {
-            if !header.is_empty() {
-                header.push(roman(", "));
-            }
-            header.push(bold(format!("--{}", long)));
-        }
-        if opt.get_action().takes_values() {
-            let value_name = &opt.get_value_names().unwrap();
-            if opt.get_long().is_some() {
-                header.push(roman("="));
-            } else {
-                header.push(roman(" "));
-            }
-
-            if opt.get_id() == "auth" {
-                header.push(italic("USER"));
-                header.push(roman("["));
-                header.push(italic(":PASS"));
-                header.push(roman("] | "));
-                header.push(italic("TOKEN"));
-            } else {
-                header.push(italic(value_name.join(" ")));
-            }
-        }
-        let mut body = vec![];
-
-        let mut help = opt
-            .get_long_help()
-            .or_else(|| opt.get_help())
-            .expect("option is missing help")
-            .to_string();
-        if !help.ends_with('.') {
-            help.push('.')
-        }
-        body.push(roman(help));
-
-        let possible_values = opt.get_possible_values();
-        if !possible_values.is_empty()
-            && !opt.is_hide_possible_values_set()
-            && opt.get_id() != "pretty"
-        {
-            let possible_values_text = format!(
-                "\n\n[possible values: {}]",
-                possible_values
-                    .iter()
-                    .map(|v| v.get_name())
-                    .collect::<Vec<_>>()
-                    .join(", ")
-            );
-            body.push(roman(possible_values_text));
-        }
-        options_roff.control("TP", ["4"]);
-        options_roff.text(header);
-        options_roff.text(body);
-    }
-
-    let mut manpage = fs::read_to_string(format!("{}/man-template.roff", rest_args[0])).unwrap();
-
-    let current_date = {
-        let (year, month, day) = DateTime::now_utc().date().to_calendar_date();
-        format!("{}-{:02}-{:02}", year, u8::from(month), day)
-    };
-
-    manpage = manpage.replace("{{date}}", &current_date);
-    manpage = manpage.replace("{{version}}", app.get_version().unwrap());
-    manpage = manpage.replace("{{request_items}}", request_items_roff.to_roff().trim());
-    manpage = manpage.replace("{{options}}", options_roff.to_roff().trim());
-
-    fs::write(format!("{}/xh.1", rest_args[0]), manpage).unwrap();
-    safe_exit();
-}
-
-#[cfg(not(feature = "man-completion-gen"))]
-fn generate_completions(mut _app: clap::Command, _rest_args: Vec<String>) -> clap::error::Error {
-    clap::Error::raw(
-        clap::error::ErrorKind::InvalidSubcommand,
-        "generate-completions requires enabling man-completion-gen feature\n",
-    )
-}
-
-#[cfg(not(feature = "man-completion-gen"))]
-fn generate_manpages(mut _app: clap::Command, _rest_args: Vec<String>) -> clap::error::Error {
-    clap::Error::raw(
-        clap::error::ErrorKind::InvalidSubcommand,
-        "generate-manpages requires enabling man-completion-gen feature\n",
-    )
-}
-
 #[derive(Default, ValueEnum, Copy, Clone, Debug, PartialEq, Eq)]
 pub enum AuthType {
     #[default]
@@ -1351,6 +1190,19 @@ pub enum HttpVersion {
     Http2,
     #[clap(name = "2-prior-knowledge")]
     Http2PriorKnowledge,
+    #[clap(name = "3-prior-knowledge")]
+    Http3PriorKnowledge,
+}
+
+#[derive(ValueEnum, Copy, Clone, Debug, PartialEq, Eq)]
+pub enum Generate {
+    CompleteBash,
+    CompleteElvish,
+    CompleteFish,
+    CompleteNushell,
+    CompletePowershell,
+    CompleteZsh,
+    Man,
 }
 
 /// HTTPie uses Python's str.decode(). That one's very accepting of different spellings.
@@ -1369,7 +1221,7 @@ pub enum HttpVersion {
 /// BE instead.
 fn parse_encoding(encoding: &str) -> anyhow::Result<&'static Encoding> {
     let normalized_encoding = encoding.to_lowercase().replace(
-        |c: char| (!c.is_alphanumeric() && c != '_' && c != '-' && c != ':'),
+        |c: char| !c.is_alphanumeric() && c != '_' && c != '-' && c != ':',
         "",
     );
 
@@ -1870,4 +1722,16 @@ mod tests {
         assert!(Resolve::from_str("example.com:::1").is_ok());
         assert!(Resolve::from_str("example.com:[::1]").is_ok());
     }
+
+    #[test]
+    fn generate() {
+        let cli = parse(["--generate", "complete-bash"]).unwrap();
+        assert_eq!(cli.generate, Some(Generate::CompleteBash));
+        assert_eq!(cli.raw_method_or_url, None);
+    }
+
+    #[test]
+    fn generate_with_url() {
+        parse(["--generate", "complete-zsh", "example.org"]).unwrap_err();
+    }
 }
