@@ -2,9 +2,12 @@
 mod auth;
 mod buffer;
 mod cli;
+mod content_disposition;
 mod decoder;
 mod download;
+mod error_reporting;
 mod formatting;
+mod generation;
 mod middleware;
 mod nested_json;
 mod netrc;
@@ -15,21 +18,22 @@ mod request_items;
 mod session;
 mod to_curl;
 mod utils;
-mod vendored;
 
 use std::env;
 use std::fs::File;
-use std::io::{self, IsTerminal, Read};
+use std::io::{self, IsTerminal, Read, Write as _};
 use std::net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr};
 use std::path::PathBuf;
-use std::process;
+use std::process::ExitCode;
 use std::str::FromStr;
 use std::sync::Arc;
 
 use anyhow::{anyhow, Context, Result};
 use cookie_store::{CookieStore, RawCookie};
+use flate2::write::ZlibEncoder;
+use hyper::header::CONTENT_ENCODING;
 use redirect::RedirectFollower;
-use reqwest::blocking::Client;
+use reqwest::blocking::{Body as ReqwestBody, Client};
 use reqwest::header::{
     HeaderValue, ACCEPT, ACCEPT_ENCODING, CONNECTION, CONTENT_TYPE, COOKIE, RANGE, USER_AGENT,
 };
@@ -46,7 +50,6 @@ use crate::printer::Printer;
 use crate::request_items::{Body, FORM_CONTENT_TYPE, JSON_ACCEPT, JSON_CONTENT_TYPE};
 use crate::session::Session;
 use crate::utils::{test_mode, test_pretend_term, url_with_query};
-use crate::vendored::reqwest_cookie_store;
 
 #[cfg(not(any(feature = "native-tls", feature = "rustls")))]
 compile_error!("Either native-tls or rustls feature must be enabled!");
@@ -60,7 +63,7 @@ fn get_user_agent() -> &'static str {
     }
 }
 
-fn main() {
+fn main() -> ExitCode {
     let args = Cli::parse();
 
     if args.debug {
@@ -73,36 +76,33 @@ fn main() {
     log::debug!("{args:#?}");
 
     let native_tls = args.native_tls;
+    let bin_name = args.bin_name.clone();
 
     match run(args) {
-        Ok(exit_code) => {
-            process::exit(exit_code);
-        }
+        Ok(exit_code) => exit_code,
         Err(err) => {
             log::debug!("{err:#?}");
-            log::error!("{err:?}");
-            let msg = err.root_cause().to_string();
-            if native_tls && msg == "invalid minimum TLS version for backend" {
+            eprintln!("{bin_name}: error: {err:?}");
+
+            for message in error_reporting::additional_messages(&err, native_tls) {
                 eprintln!();
-                eprintln!("Try running without the --native-tls flag.");
+                eprintln!("{message}");
             }
-            if let Some(err) = err.downcast_ref::<reqwest::Error>() {
-                if err.is_timeout() {
-                    process::exit(2);
-                }
-            }
-            if msg.starts_with("Too many redirects") {
-                process::exit(6);
-            }
-            process::exit(1);
+
+            error_reporting::exit_code(&err)
         }
     }
 }
 
-fn run(args: Cli) -> Result<i32> {
+fn run(args: Cli) -> Result<ExitCode> {
+    if let Some(generate) = args.generate {
+        generation::generate(&args.bin_name, generate);
+        return Ok(ExitCode::SUCCESS);
+    }
+
     if args.curl {
         to_curl::print_curl_translation(args)?;
-        return Ok(0);
+        return Ok(ExitCode::SUCCESS);
     }
 
     let (mut headers, headers_to_unset) = args.request_items.headers()?;
@@ -182,7 +182,7 @@ fn run(args: Cli) -> Result<i32> {
         return Err(anyhow!("This binary was built without native-tls support"));
     }
 
-    let mut exit_code: i32 = 0;
+    let mut failure_code = None;
     let mut resume: Option<u64> = None;
     let mut auth = None;
     let mut save_auth_in_session = true;
@@ -278,6 +278,21 @@ fn run(args: Cli) -> Result<i32> {
         Some(HttpVersion::Http10 | HttpVersion::Http11) => client.http1_only(),
         Some(HttpVersion::Http2PriorKnowledge) => client.http2_prior_knowledge(),
         Some(HttpVersion::Http2) => client,
+        Some(HttpVersion::Http3PriorKnowledge) => {
+            #[cfg(feature = "http3")]
+            {
+                if args.native_tls {
+                    return Err(anyhow!("HTTP/3 is not supported when using native-tls"));
+                }
+                client.http3_prior_knowledge()
+            }
+            #[cfg(not(feature = "http3"))]
+            {
+                return Err(anyhow!(
+                    "This binary was built without support for HTTP/3. Enable the `http3` feature."
+                ));
+            }
+        }
         None => client,
     };
 
@@ -294,12 +309,34 @@ fn run(args: Cli) -> Result<i32> {
         if let Ok(ip_addr) = IpAddr::from_str(name_or_ip) {
             client = client.local_address(ip_addr);
         } else {
-            #[cfg(any(target_os = "android", target_os = "fuchsia", target_os = "linux"))]
+            #[cfg(any(
+                target_os = "android",
+                target_os = "fuchsia",
+                target_os = "illumos",
+                target_os = "ios",
+                target_os = "linux",
+                target_os = "macos",
+                target_os = "solaris",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
+            ))]
             {
                 client = client.interface(name_or_ip);
             }
 
-            #[cfg(not(any(target_os = "android", target_os = "fuchsia", target_os = "linux")))]
+            #[cfg(not(any(
+                target_os = "android",
+                target_os = "fuchsia",
+                target_os = "illumos",
+                target_os = "ios",
+                target_os = "linux",
+                target_os = "macos",
+                target_os = "solaris",
+                target_os = "tvos",
+                target_os = "visionos",
+                target_os = "watchos",
+            )))]
             {
                 #[cfg(not(feature = "network-interface"))]
                 return Err(anyhow!(
@@ -327,6 +364,18 @@ fn run(args: Cli) -> Result<i32> {
         };
     }
 
+    #[cfg(unix)]
+    if let Some(socket_path) = args.unix_socket {
+        client = client.unix_socket(socket_path);
+    }
+
+    #[cfg(not(unix))]
+    if args.unix_socket.is_some() {
+        return Err(anyhow::anyhow!(
+            "--unix-socket is not supported on this platform"
+        ));
+    }
+
     for resolve in args.resolve {
         client = client.resolve(&resolve.domain, SocketAddr::new(resolve.addr, 0));
     }
@@ -389,6 +438,9 @@ fn run(args: Cli) -> Result<i32> {
             Some(HttpVersion::Http2 | HttpVersion::Http2PriorKnowledge) => {
                 request_builder.version(reqwest::Version::HTTP_2)
             }
+            Some(HttpVersion::Http3PriorKnowledge) => {
+                request_builder.version(reqwest::Version::HTTP_3)
+            }
             None => request_builder,
         };
 
@@ -444,8 +496,27 @@ fn run(args: Cli) -> Result<i32> {
         };
 
         if args.resume {
-            if let Some(file_size) = get_file_size(args.output.as_deref()) {
-                request_builder = request_builder.header(RANGE, format!("bytes={}-", file_size));
+            if headers.contains_key(RANGE) {
+                // There are no good options here, and `--continue` works on a
+                // best-effort basis, so give up with a warning.
+                //
+                // - HTTPie:
+                //   - If the file does not exist, errors when the response has
+                //     an apparently incorrect content range, as though it sent
+                //     `Range: bytes=0-`.
+                //   - If the file already exists, ignores the manual header
+                //     and downloads what's probably the wrong data.
+                // - wget gives priority to the manual header and keeps failing
+                //   and retrying the download (with or without existing file).
+                // - curl gives priority to the manual header and reports that
+                //   the server does not support partial downloads. It also has
+                //   a --range CLI option which is mutually exclusive with its
+                //   --continue-at option.
+                log::warn!(
+                    "--continue can't be used with a 'Range:' header. --continue will be disabled."
+                );
+            } else if let Some(file_size) = get_file_size(args.output.as_deref()) {
+                request_builder = request_builder.header(RANGE, format!("bytes={file_size}-"));
                 resume = Some(file_size);
             }
         }
@@ -484,6 +555,25 @@ fn run(args: Cli) -> Result<i32> {
 
         let mut request = request_builder.headers(headers).build()?;
 
+        if args.compress >= 1 {
+            if request.headers().contains_key(CONTENT_ENCODING) {
+                // HTTPie overrides the original Content-Encoding header in this case
+                log::warn!("--compress can't be used with a 'Content-Encoding:' header. --compress will be disabled.");
+            } else if let Some(body) = request.body_mut() {
+                // TODO: Compress file body (File) without buffering
+                let body_bytes = body.buffer()?;
+                let mut encoder = ZlibEncoder::new(Vec::new(), Default::default());
+                encoder.write_all(body_bytes)?;
+                let output = encoder.finish()?;
+                if output.len() < body_bytes.len() || args.compress >= 2 {
+                    *body = ReqwestBody::from(output);
+                    request
+                        .headers_mut()
+                        .insert(CONTENT_ENCODING, HeaderValue::from_static("deflate"));
+                }
+            }
+        }
+
         for header in &headers_to_unset {
             request.headers_mut().remove(header);
         }
@@ -577,16 +667,17 @@ fn run(args: Cli) -> Result<i32> {
 
         let status = response.status();
         if args.check_status.unwrap_or(!args.httpie_compat_mode) {
-            exit_code = match status.as_u16() {
-                300..=399 if !args.follow => 3,
-                400..=499 => 4,
-                500..=599 => 5,
-                _ => 0,
-            };
+            match status.as_u16() {
+                300..=399 if !args.follow => failure_code = Some(ExitCode::from(3)),
+                400..=499 => failure_code = Some(ExitCode::from(4)),
+                500..=599 => failure_code = Some(ExitCode::from(5)),
+                _ => (),
+            }
+
             // Print this if the status code isn't otherwise ending up in the terminal.
             // HTTPie looks at --quiet, since --quiet always suppresses the response
             // headers even if you pass --print=h. But --print takes precedence for us.
-            if exit_code != 0 && (is_output_redirected || !print.response_headers) {
+            if failure_code.is_some() && (is_output_redirected || !print.response_headers) {
                 log::warn!("HTTP {} {}", status.as_u16(), reason_phrase(&response));
             }
         }
@@ -595,7 +686,7 @@ fn run(args: Cli) -> Result<i32> {
             printer.print_response_headers(&response)?;
         }
         if args.download {
-            if exit_code == 0 {
+            if failure_code.is_none() {
                 download_file(
                     response,
                     args.output,
@@ -625,7 +716,7 @@ fn run(args: Cli) -> Result<i32> {
             .with_context(|| format!("couldn't persist session {}", s.path.display()))?;
     }
 
-    Ok(exit_code)
+    Ok(failure_code.unwrap_or(ExitCode::SUCCESS))
 }
 
 /// Configure backtraces for standard panics and anyhow using `$RUST_BACKTRACE`.
