@@ -90,8 +90,7 @@ impl FromStr for RequestItem {
                         clap::Error::raw(
                             clap::error::ErrorKind::InvalidValue,
                             format!(
-                                "Invalid value for '[REQUEST_ITEM]...': {:?} {}",
-                                request_item, err
+                                "Invalid value for '[REQUEST_ITEM]...': {request_item:?} {err}"
                             ),
                         )
                     })?,
@@ -130,7 +129,7 @@ impl FromStr for RequestItem {
             // item
             Err(clap::Error::raw(
                 clap::error::ErrorKind::InvalidValue,
-                format!("Invalid value for '[REQUEST_ITEM]...': {:?}", request_item),
+                format!("Invalid value for '[REQUEST_ITEM]...': {request_item:?}"),
             ))
         }
     }
@@ -220,7 +219,6 @@ pub enum Body {
     File {
         file_name: PathBuf,
         file_type: Option<HeaderValue>,
-        #[allow(dead_code)]
         file_name_header: Option<String>,
     },
 }
@@ -294,7 +292,7 @@ impl RequestItems {
         Ok((headers, headers_to_unset))
     }
 
-    pub fn query(&self) -> Result<Vec<(&str, Cow<str>)>> {
+    pub fn query(&self) -> Result<Vec<(&str, Cow<'_, str>)>> {
         let mut query: Vec<(&str, Cow<str>)> = vec![];
         for item in &self.items {
             if let RequestItem::UrlParam(key, value) = item {
