@@ -177,11 +177,13 @@ impl Session {
             path
         };
 
+        log::debug!("Checking for session in {path:?}");
         let content = match fs::read_to_string(&path) {
             Ok(content) => serde_json::from_str::<Content>(&content)?.migrate(),
             Err(err) if err.kind() == io::ErrorKind::NotFound => Content::default(),
             Err(err) => return Err(err.into()),
         };
+        log::debug!("Loaded session from {path:?}");
 
         Ok(Session {
             url,
@@ -256,13 +258,13 @@ impl Session {
                 let password = password.as_deref().unwrap_or("");
                 self.content.auth = Auth {
                     auth_type: Some("basic".into()),
-                    raw_auth: Some(format!("{}:{}", username, password)),
+                    raw_auth: Some(format!("{username}:{password}")),
                 }
             }
             auth::Auth::Digest(username, password) => {
                 self.content.auth = Auth {
                     auth_type: Some("digest".into()),
-                    raw_auth: Some(format!("{}:{}", username, password)),
+                    raw_auth: Some(format!("{username}:{password}")),
                 }
             }
             auth::Auth::Bearer(token) => {
@@ -279,7 +281,7 @@ impl Session {
             Cookies::Map(_) => unreachable!(),
             Cookies::List(cookies) => cookies.iter().map(|cookie| {
                 let mut cookie_builder =
-                    cookie_store::RawCookie::build(cookie.name.clone(), cookie.value.clone());
+                    cookie_store::RawCookie::build((cookie.name.clone(), cookie.value.clone()));
 
                 if let Some(expires) = cookie.expires {
                     cookie_builder =
@@ -303,7 +305,7 @@ impl Session {
                 }
 
                 Ok(cookie_store::Cookie::try_from_raw_cookie(
-                    &cookie_builder.finish(),
+                    &cookie_builder.into(),
                     &cookie_url,
                 )?)
             }),
@@ -334,7 +336,7 @@ impl Session {
                     .expires()
                     .and_then(|v| v.datetime())
                     .map(|v| v.unix_timestamp()),
-                path: cookie.path().map(Into::into),
+                path: Some(cookie.path.to_string()),
                 secure: cookie.secure(),
                 domain: domain.map(Into::into),
             });
@@ -347,6 +349,7 @@ impl Session {
                 fs::create_dir_all(parent_path)?;
             }
             let mut session_file = fs::File::create(&self.path)?;
+            log::debug!("Persisting session to {:?}", self.path);
             let formatter = serde_json::ser::PrettyFormatter::with_indent(b"    ");
             let mut ser = serde_json::Serializer::with_formatter(&mut session_file, formatter);
             self.content.serialize(&mut ser)?;
@@ -373,7 +376,7 @@ fn path_from_url(url: &Url) -> Result<String> {
         (Some("."), _) | (Some(".."), _) | (None, _) => {
             Err(anyhow!("couldn't extract host from url"))
         }
-        (Some(host), Some(port)) => Ok(format!("{}_{}", host, port)),
+        (Some(host), Some(port)) => Ok(format!("{host}_{port}")),
         (Some(host), None) => Ok(host.into()),
     }
 }
