@@ -13,12 +13,12 @@ pub fn print_curl_translation(args: Cli) -> Result<()> {
     let cmd = translate(args)?;
     let mut stderr = stderr();
     for warning in &cmd.warnings {
-        writeln!(stderr, "Warning: {}", warning)?;
+        writeln!(stderr, "Warning: {warning}")?;
     }
     if !cmd.warnings.is_empty() {
         writeln!(stderr)?;
     }
-    writeln!(stdout(), "{}", cmd)?;
+    writeln!(stdout(), "{cmd}")?;
     Ok(())
 }
 
@@ -53,7 +53,7 @@ impl Command {
 
     fn header(&mut self, name: &str, value: &str) {
         self.opt("-H", "--header");
-        self.arg(format!("{}: {}", name, value));
+        self.arg(format!("{name}: {value}"));
     }
 
     fn env(&mut self, var: &'static str, value: impl Into<String>) {
@@ -97,6 +97,8 @@ pub fn translate(args: Cli) -> Result<Command> {
         // No equivalent
         (args.style.is_some(), "-s/--style"),
         // No equivalent
+        (args.compress > 0, "-x/--compress"),
+        // No equivalent
         (args.response_charset.is_some(), "--response-charset"),
         // No equivalent
         (args.response_mime.is_some(), "--response-mime"),
@@ -113,7 +115,7 @@ pub fn translate(args: Cli) -> Result<Command> {
 
     for (present, flag) in ignored {
         if present {
-            cmd.warn(format!("Ignored {}", flag));
+            cmd.warn(format!("Ignored {flag}"));
         }
     }
 
@@ -241,6 +243,7 @@ pub fn translate(args: Cli) -> Result<Command> {
             HttpVersion::Http11 => cmd.arg("--http1.1"),
             HttpVersion::Http2 => cmd.arg("--http2"),
             HttpVersion::Http2PriorKnowledge => cmd.arg("--http2-prior-knowledge"),
+            HttpVersion::Http3PriorKnowledge => cmd.arg("--http3-only"),
         }
     }
 
@@ -299,6 +302,11 @@ pub fn translate(args: Cli) -> Result<Command> {
         cmd.arg(interface);
     };
 
+    if let Some(unix_socket) = args.unix_socket {
+        cmd.arg("--unix-socket");
+        cmd.arg(unix_socket);
+    }
+
     if !args.resolve.is_empty() {
         let port = url
             .port_or_known_default()
@@ -315,14 +323,14 @@ pub fn translate(args: Cli) -> Result<Command> {
     for (header, value) in headers.iter() {
         cmd.opt("-H", "--header");
         if value.is_empty() {
-            cmd.arg(format!("{};", header));
+            cmd.arg(format!("{header};"));
         } else {
             cmd.arg(format!("{}: {}", header, value.to_utf8_str()?));
         }
     }
     for header in headers_to_unset {
         cmd.opt("-H", "--header");
-        cmd.arg(format!("{}:", header));
+        cmd.arg(format!("{header}:"));
     }
     if args.ignore_netrc {
         // Already the default, so a bit questionable
@@ -369,11 +377,11 @@ pub fn translate(args: Cli) -> Result<Command> {
                 }
                 RequestItem::DataField { key, value, .. } => {
                     cmd.opt("-F", "--form");
-                    cmd.arg(format!("{}={}", key, value));
+                    cmd.arg(format!("{key}={value}"));
                 }
                 RequestItem::DataFieldFromFile { key, value, .. } => {
                     cmd.opt("-F", "--form");
-                    cmd.arg(format!("{}=<{}", key, value));
+                    cmd.arg(format!("{key}=<{value}"));
                 }
                 RequestItem::FormFile {
                     key,
@@ -382,7 +390,7 @@ pub fn translate(args: Cli) -> Result<Command> {
                     file_name_header,
                 } => {
                     cmd.opt("-F", "--form");
-                    let mut val = format!("{}=@{}", key, file_name);
+                    let mut val = format!("{key}=@{file_name}");
                     if let Some(file_type) = file_type {
                         val.push_str(";type=");
                         val.push_str(&file_type);
@@ -562,7 +570,7 @@ mod tests {
         for (input, output) in expected {
             let cli = Cli::try_parse_from(input.split_whitespace()).unwrap();
             let cmd = translate(cli).unwrap();
-            assert_eq!(cmd.to_string(), output, "Wrong output for {:?}", input);
+            assert_eq!(cmd.to_string(), output, "Wrong output for {input:?}");
         }
     }
 }
